﻿using System;
using System.Net;
using System.Windows;
using System.Drawing;
using System.Windows.Documents;
using System.Windows.Media;
using System.Windows.Controls;

/*
 *         Progetto: Chat P2P basata su comunicazioni UDP e TCP
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 3/11/2015
 *  
 *      Descrizione: Routine di gestione dell'interfaccia grafica.
 *                   Include la funzione di avvio del TCP Listener.    
 *                   
 */

namespace ChatWPF
{
    public class Info
    {
        public string IndirizzoIP { get; set; }
        public string NomeUtente { get; set; }

        public override string ToString()
        {
            return String.Format("{0} ({1})", NomeUtente, IndirizzoIP);
        }
    }

    /// <summary>
    /// Logica di interazione per MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        UtenteDialog ug = new UtenteDialog();

        public MainWindow()
        {
            InitializeComponent();
        }

        // Inserisce oppure aggiorna un utente nella ComboBox

        private void AggiungiNuovaInfo(string indirizzo, string utente)
        {
            bool trovato = false;
            foreach (Info i in elenco.Items)
            {
                if (i.IndirizzoIP.Equals(indirizzo))
                {
                    // Individuata la riga da aggiornare
                    i.NomeUtente = utente;
                    trovato = true;
                    break;
                }
            }

            if (!trovato)
                elenco.Items.Add(new Info() { IndirizzoIP = indirizzo, NomeUtente = utente });

            if (elenco.SelectedIndex < 0)
                elenco.SelectedIndex = 0;
            elenco.Items.Refresh();     // Mostra gli aggiornamenti effettuati sullo schermo
        }

        // Rimuove un utente dalla CombBox

        private void RimuoviInfo(string indirizzo)
        {

            foreach (Info i in elenco.Items)
            {
                if (i.IndirizzoIP.Equals(indirizzo))
                {
                    elenco.Items.Remove(i);
                    break;
                }
            }

            elenco.Items.Refresh();
        }

        private void Scrivi(string testo, SolidColorBrush colore)
        {
            Paragraph p = new Paragraph(new Run(testo));
            p.Foreground = colore;
            p.Margin = new Thickness(3);
            rtbOutput.Document.Blocks.Add(p);
            rtbOutput.ScrollToEnd();
        }

        private void ElaboraDatiRicevutiTcp(IPEndPoint ep, string msg)
        {
            //MessageBox.Show("TCP " + ep.ToString() + " " + msg);
            string utenteRemoto = null;

            foreach (Info i in elenco.Items)
            {
                if (i.IndirizzoIP.Equals(ep.Address.ToString())) {
                    utenteRemoto = i.NomeUtente;
                    break;
                }
            }

            if (utenteRemoto == null)
                utenteRemoto = "<Utente sconosciuto>";

            Scrivi(utenteRemoto + ": " + msg, Brushes.Black);
        }

        private void ElaboraDatiRicevutiUdp(IPEndPoint hostRemoto, string messaggio)
        {
            const char separatore = '|';
            // Scompone il messaggio nelle parti necessarie al completamento dell'elaborazione

            string[] parti = messaggio.Split(new char[] { separatore});
            string azione = parti.Length > 0 ? parti[0].ToUpper() : "";
            string utenteRemoto = parti.Length > 1 ? parti[1] : "";

            if (azione.Equals("NUOVO"))
            {
                // Occorre aggiungere il nuovo utente remoto alla ListView e inviargli un messaggio di 
                // risposta contenente il nome utente locale

                AggiungiNuovaInfo(hostRemoto.Address.ToString(), utenteRemoto);

                //string utenteLocale = (faseModificaNome == false ? tbUtente.Text : "<Non disponibile>");
                Utility.InviaMessaggioUdp(hostRemoto.Address, "RISPOSTA" + separatore + Utility.UtenteLocale);

            }
            else if (azione.Equals("RISPOSTA"))
            {
                // La risposta fornita dall'host remoto permette di aggiornare l'elenco degli utenti connessi
                AggiungiNuovaInfo(hostRemoto.Address.ToString(), utenteRemoto);
            }
            else if (azione.Equals("CHIUDI"))
            {
                // Occorre eliminare dalla ListView l'utente che ha segnalato la chiusura del collegamento.
                RimuoviInfo(hostRemoto.Address.ToString());
            }

        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {

            Utility.UtenteLocale = ug.NomeUtente;
            Utility.IndirizzoLocale = ug.IndirizzoIp;
            Utility.Broadcast = ug.Broadcast;

            lbUtenteLocale.Content = String.Format("{0} ({1})", Utility.UtenteLocale, Utility.IndirizzoLocale);

            if (Utility.InizializzaListener())
            {
                Utility.AvviaListenerUdp(new Action<IPEndPoint, string>(this.ElaboraDatiRicevutiUdp));
                Utility.AvviaListenerTcp(new Action<IPEndPoint, string>(this.ElaboraDatiRicevutiTcp));

                //Utility.InviaMessaggioUdp(Utility.Broadcast, "NUOVO|" + Utility.UtenteLocale);
                for (int i = 0; i < 4; i++)
                {
                    Utility.InviaMessaggioUdp(Utility.Broadcast, "NUOVO|" + Utility.UtenteLocale);
                    System.Threading.Thread.Sleep(100);
                }
            }
            else
            {
                Application.Current.Shutdown();
            }
        }

        private void btnInvia_Click(object sender, RoutedEventArgs e)
        {
            int idx = elenco.SelectedIndex;

            if (idx < 0)
                return;
            Info info = (Info)elenco.Items[idx];

            Utility.InviaMessaggioTcpAsync(IPAddress.Parse(info.IndirizzoIP), tbMessaggio.Text);
            Scrivi("TU: " + tbMessaggio.Text, Brushes.Blue);
            tbMessaggio.Text = "";
        }

        private void Window_Initialized(object sender, EventArgs e)
        {

            if (ug.ShowDialog() == false)
            {
                Close();
                return;
            }

        }

        private void Window_Closed(object sender, EventArgs e)
        {
            if (Utility.ListUdp != null)
            {
                Utility.InviaMessaggioUdp(Utility.Broadcast, "CHIUDI");
                Utility.ListUdp.Close();
            }

            if (Utility.ListTcp != null)
                Utility.ListTcp.Stop();
        }

        private void tbMessaggio_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {
            btnInvia.IsEnabled = ((TextBox)sender).Text.Length > 0;
        }

        private void miRimuoviContenuto_Click(object sender, RoutedEventArgs e)
        {
            rtbOutput.Document.Blocks.Clear();
        }

    }
}
